@@ -313,6 +313,7 @@ class Dbaas(object):
         from troveclient.v1 import limits
         from troveclient.v1 import management
         from troveclient.v1 import metadata
+        from troveclient.v1 import modules
         from troveclient.v1 import quota
         from troveclient.v1 import root
         from troveclient.v1 import security_groups
@@ -354,6 +355,7 @@ class Dbaas(object):
         config_parameters = configurations.ConfigurationParameters(self)
         self.configuration_parameters = config_parameters
         self.metadata = metadata.Metadata(self)
+        self.modules = modules.Modules(self)
         self.mgmt_configs = management.MgmtConfigurationParameters(self)
         self.mgmt_datastore_versions = management.MgmtDatastoreVersions(self)
 
