@@ -34,3 +34,8 @@ class NoTokenLookupException(Exception):
 class ResponseFormatError(Exception):
     """Could not parse the response format."""
     pass
+
+
+class GuestLogNotFoundError(Exception):
+    """The specified guest log does not exist."""
+    pass
