@@ -174,38 +174,41 @@ class ShellTest(utils.TestCase):
         self.assert_called('DELETE', '/clusters/cls-1234')
 
     def test_boot(self):
-        self.run_command('create test-member-1 1 --size 1')
+        self.run_command('create test-member-1 1 --size 1 --volume_type lvm')
         self.assert_called_anytime(
             'POST', '/instances',
             {'instance': {
-                'volume': {'size': 1},
+                'volume': {'size': 1, 'type': 'lvm'},
                 'flavorRef': 1,
                 'name': 'test-member-1',
                 'replica_count': 1
             }})
 
     def test_boot_by_flavor_name(self):
-        self.run_command('create test-member-1 m1.tiny --size 1')
+        self.run_command(
+            'create test-member-1 m1.tiny --size 1 --volume_type lvm')
         self.assert_called_anytime(
             'POST', '/instances',
             {'instance': {
-                'volume': {'size': 1},
+                'volume': {'size': 1, 'type': 'lvm'},
                 'flavorRef': 1,
                 'name': 'test-member-1',
                 'replica_count': 1
             }})
 
     def test_boot_nic_error(self):
-        cmd = ('create test-member-1 1 --size 1 '
+        cmd = ('create test-member-1 1 --size 1 --volume_type lvm '
                '--nic net-id=some-id,port-id=some-id')
         self.assertRaisesRegexp(
-            exceptions.ValidationError, 'Invalid nic argument',
+            exceptions.ValidationError,
+            'Invalid NIC argument: nic=\'net-id=some-id,port-id=some-id\'',
             self.run_command, cmd)
 
     def test_cluster_create(self):
         cmd = ('cluster-create test-clstr vertica 7.1 '
                '--instance flavor=2,volume=2 '
-               '--instance flavor=2,volume=1')
+               '--instance flavor=2,volume=1 '
+               '--instance flavor=2,volume=1,volume_type=my-type-1')
         self.run_command(cmd)
         self.assert_called_anytime(
             'POST', '/clusters',
@@ -218,6 +221,10 @@ class ShellTest(utils.TestCase):
                     {
                         'volume': {'size': '1'},
                         'flavorRef': '2'
+                    },
+                    {
+                        'volume': {'size': '1', 'type': 'my-type-1'},
+                        'flavorRef': '2'
                     }],
                 'datastore': {'version': '7.1', 'type': 'vertica'},
                 'name': 'test-clstr'}})
@@ -246,7 +253,7 @@ class ShellTest(utils.TestCase):
         cmd = ('cluster-create test-clstr vertica 7.1 --instance volume=2 '
                '--instance flavor=2,volume=1')
         self.assertRaisesRegexp(
-            exceptions.ValidationError, 'flavor is required',
+            exceptions.MissingArgs, 'Missing argument\(s\): flavor',
             self.run_command, cmd)
 
     def test_cluster_grow(self):
@@ -261,6 +268,53 @@ class ShellTest(utils.TestCase):
         self.run_command(cmd)
         self.assert_called('POST', '/clusters/cls-1234')
 
+    def test_cluster_create_with_nic_az(self):
+        cmd = ('cluster-create test-clstr1 vertica 7.1 '
+               '--instance flavor=2,volume=2,nic=\'net-id=some-id\','
+               'availability_zone=2 '
+               '--instance flavor=2,volume=2,nic=\'net-id=some-id\','
+               'availability_zone=2')
+        self.run_command(cmd)
+        self.assert_called_anytime(
+            'POST', '/clusters',
+            {'cluster': {
+                'instances': [
+                    {
+                        'flavorRef': '2',
+                        'volume': {'size': '2'},
+                        'nics': [{'net-id': 'some-id'}],
+                        'availability_zone': '2'
+                    },
+                    {
+                        'flavorRef': '2',
+                        'volume': {'size': '2'},
+                        'nics': [{'net-id': 'some-id'}],
+                        'availability_zone': '2'
+                    }],
+                'datastore': {'version': '7.1', 'type': 'vertica'},
+                'name': 'test-clstr1'}})
+
+    def test_cluster_create_with_nic_az_error(self):
+        cmd = ('cluster-create test-clstr vertica 7.1 '
+               '--instance flavor=2,volume=2,nic=net-id=some-id,'
+               'port-id=some-port-id,availability_zone=2 '
+               '--instance flavor=2,volume=1,nic=net-id=some-id,'
+               'port-id=some-port-id,availability_zone=2')
+        self.assertRaisesRegexp(
+            exceptions.ValidationError, "Invalid 'nic' parameter. "
+            "The value must be quoted.",
+            self.run_command, cmd)
+
+    def test_cluster_create_with_nic_az_error_again(self):
+        cmd = ('cluster-create test-clstr vertica 7.1 '
+               '--instance flavor=2,volume=2,nic=\'v4-fixed-ip=10.0.0.1\','
+               'availability_zone=2 '
+               '--instance flavor=2,volume=1,nic=\'v4-fixed-ip=10.0.0.1\','
+               'availability_zone=2')
+        self.assertRaisesRegexp(
+            exceptions.ValidationError, 'Invalid NIC argument',
+            self.run_command, cmd)
+
     def test_datastore_list(self):
         self.run_command('datastore-list')
         self.assert_called('GET', '/datastores')
@@ -485,6 +539,10 @@ class ShellTest(utils.TestCase):
         self.run_command_clusters('root-enable cls-1234')
         self.assert_called_anytime('POST', '/clusters/cls-1234/root')
 
+    def test_root_disable_instance(self):
+        self.run_command('root-disable 1234')
+        self.assert_called_anytime('DELETE', '/instances/1234/root')
+
     def test_root_show_instance(self):
         self.run_command('root-show 1234')
         self.assert_called('GET', '/instances/1234/root')
