@@ -16,11 +16,15 @@
 
 from __future__ import print_function
 
+import argparse
 import sys
 import time
 
 INSTANCE_ERROR = ("Instance argument(s) must be of the form --instance "
-                  "<flavor=flavor_name_or_id, volume=volume>")
+                  "<opt=value[,opt=value]> - see help for details.")
+NIC_ERROR = ("Invalid NIC argument: %s. Must specify either net-id or port-id "
+             "but not both. Please refer to help.")
+NO_LOG_FOUND_ERROR = "ERROR: No published '%s' log was found for %s."
 
 try:
     import simplejson as json
@@ -135,6 +139,21 @@ def _find_backup(cs, backup):
     return utils.find_resource(cs.backups, backup)
 
 
+def _find_module(cs, module):
+    """Get a module by ID."""
+    return utils.find_resource(cs.modules, module)
+
+
+def _find_datastore(cs, datastore):
+    """Get a datastore by ID."""
+    return utils.find_resource(cs.datastores, datastore)
+
+
+def _find_datastore_version(cs, datastore_version):
+    """Get a datastore version by ID."""
+    return utils.find_resource(cs.datastores, datastore_version)
+
+
 # Flavor related calls
 @utils.arg('--datastore_type', metavar='<datastore_type>',
            default=None,
@@ -379,6 +398,11 @@ def do_update(cs, args):
            default=None,
            help="Size of the instance disk volume in GB. "
                 "Required when volume support is enabled.")
+@utils.arg('--volume_type',
+           metavar='<volume_type>',
+           type=str,
+           default=None,
+           help="Volume type. Optional when volume support is enabled.")
 @utils.arg('flavor',
            metavar='<flavor>',
            help='Flavor ID or name of the instance.')
@@ -436,7 +460,8 @@ def do_create(cs, args):
     replica_of_instance = None
     flavor_id = _find_flavor(cs, args.flavor).id
     if args.size:
-        volume = {"size": args.size}
+        volume = {"size": args.size,
+                  "type": args.volume_type}
     restore_point = None
     if args.backup:
         restore_point = {"backupRef": args.backup}
@@ -449,12 +474,7 @@ def do_create(cs, args):
     for nic_str in args.nics:
         nic_info = dict([(k, v) for (k, v) in [z.split("=", 1)[:2] for z in
                                                nic_str.split(",")]])
-        if bool(nic_info.get('net-id')) == bool(nic_info.get('port-id')):
-            err_msg = ("Invalid nic argument '%s'. Nic arguments must be of "
-                       "the form --nic <net-id=net-uuid,v4-fixed-ip=ip-addr,"
-                       "port-id=port-uuid>, with at minimum net-id or port-id "
-                       "(but not both) specified." % nic_str)
-            raise exceptions.ValidationError(err_msg)
+        _validate_nic_info(nic_info, nic_str)
         nics.append(nic_info)
 
     instance = cs.instances.create(args.name,
@@ -473,41 +493,135 @@ def do_create(cs, args):
     _print_instance(instance)
 
 
+def _validate_nic_info(nic_info, nic_str):
+    # need one or the other, not both, not none (!= ~ XOR)
+    if not (bool(nic_info.get('net-id')) != bool(nic_info.get('port-id'))):
+        raise exceptions.ValidationError(NIC_ERROR % ("nic='%s'" % nic_str))
+
+
+def _get_flavors(cs, instance_str):
+    flavor_name = _get_instance_property(instance_str, 'flavor', True)
+    flavor_id = _find_flavor(cs, flavor_name).id
+    return str(flavor_id)
+
+
+def _get_networks(instance_str):
+    nic_args = _dequote(_get_instance_property(instance_str, 'nic',
+                                               is_required=False, quoted=True))
+
+    nic_info = {}
+    if nic_args:
+        net_id = _get_instance_property(nic_args, 'net-id', False)
+        port_id = _get_instance_property(nic_args, 'port-id', False)
+        fixed_ipv4 = _get_instance_property(nic_args, 'v4-fixed-ip', False)
+
+        if net_id:
+            nic_info.update({'net-id': net_id})
+        if port_id:
+            nic_info.update({'port-id': port_id})
+        if fixed_ipv4:
+            nic_info.update({'v4-fixed-ip': fixed_ipv4})
+
+        _validate_nic_info(nic_info, nic_args)
+        return [nic_info]
+
+    return None
+
+
+def _dequote(value):
+    def _strip_quotes(value, quote_char):
+        if value:
+            return value.strip(quote_char)
+        return value
+
+    return _strip_quotes(_strip_quotes(value, "'"), '"')
+
+
+def _get_volumes(instance_str):
+    volume_size = _get_instance_property(instance_str, 'volume', True)
+    volume_type = _get_instance_property(instance_str, 'volume_type', False)
+
+    volume_info = {"size": volume_size}
+    if volume_type:
+        volume_info.update({"type": volume_type})
+
+    return volume_info
+
+
+def _get_availability_zones(instance_str):
+    return _get_instance_property(instance_str, 'availability_zone', False)
+
+
+def _get_instance_property(instance_str, property_name, is_required=True,
+                           quoted=False):
+    if property_name in instance_str:
+        try:
+            left = instance_str.split('%s=' % property_name)[1]
+
+            # Handle complex (quoted) properties. Strip the quotes.
+            quote = left[0]
+            if quote in ["'", '"']:
+                left = left[1:]
+            else:
+                if quoted:
+                    # Fail if quotes are required.
+                    raise exceptions.ValidationError(
+                        "Invalid '%s' parameter. The value must be quoted."
+                        % property_name)
+                quote = ''
+
+            property_value = left.split('%s,' % quote)[0]
+            return str(property_value).strip()
+        except IndexError:
+            raise exceptions.ValidationError("Invalid '%s' parameter. %s."
+                                             % (property_name, INSTANCE_ERROR))
+
+    if is_required:
+        raise exceptions.MissingArgs([property_name])
+
+    return None
+
+
 @utils.arg('name',
            metavar='<name>',
            type=str,
            help='Name of the cluster.')
 @utils.arg('datastore',
            metavar='<datastore>',
-           help='A datastore name or UUID.')
+           help='A datastore name or ID.')
 @utils.arg('datastore_version',
            metavar='<datastore_version>',
-           help='A datastore version name or UUID.')
+           help='A datastore version name or ID.')
 @utils.arg('--instance',
-           metavar="<flavor=flavor_name_or_id,volume=volume>",
+           metavar='"<opt=value,opt=value,...>"',
+           help="Create an instance for the cluster.  Specify multiple "
+                "times to create multiple instances.  "
+                "Valid options are: flavor=flavor_name_or_id, "
+                "volume=disk_size_in_GB, volume_type=type, "
+                "nic='net-id=net-uuid,v4-fixed-ip=ip-addr,port-id=port-uuid' "
+                "(where net-id=network_id, v4-fixed-ip=IPv4r_fixed_address, "
+                "port-id=port_id), availability_zone=AZ_hint_for_Nova.",
            action='append',
            dest='instances',
-           default=[],
-           help="Create an instance for the cluster.  Specify multiple "
-                "times to create multiple instances.")
+           default=[])
 @utils.service_type('database')
 def do_cluster_create(cs, args):
     """Creates a new cluster."""
     instances = []
     for instance_str in args.instances:
         instance_info = {}
-        for z in instance_str.split(","):
-            for (k, v) in [z.split("=", 1)[:2]]:
-                if k == "flavor":
-                    flavor_id = _find_flavor(cs, v).id
-                    instance_info["flavorRef"] = str(flavor_id)
-                elif k == "volume":
-                    instance_info["volume"] = {"size": v}
-                else:
-                    instance_info[k] = v
-        if not instance_info.get('flavorRef'):
-            err_msg = ("flavor is required. %s." % INSTANCE_ERROR)
-            raise exceptions.ValidationError(err_msg)
+
+        instance_info["flavorRef"] = _get_flavors(cs, instance_str)
+        instance_info["volume"] = _get_volumes(instance_str)
+
+        nics = _get_networks(instance_str)
+        if nics:
+            instance_info["nics"] = nics
+
+        availability_zones = _get_availability_zones(instance_str)
+        if availability_zones:
+            instance_info["availability_zone"] = availability_zones
+
         instances.append(instance_info)
 
     if len(instances) == 0:
@@ -658,11 +772,12 @@ def do_backup_list(cs, args):
                      order_by='updated')
 
 
-@utils.arg('backup', metavar='<backup>', help='ID of the backup.')
+@utils.arg('backup', metavar='<backup>', help='ID or name of the backup.')
 @utils.service_type('database')
 def do_backup_delete(cs, args):
     """Deletes a backup."""
-    cs.backups.delete(args.backup)
+    backup = _find_backup(cs, args.backup)
+    cs.backups.delete(backup)
 
 
 @utils.arg('instance', metavar='<instance>',
@@ -926,6 +1041,15 @@ def do_root_enable(cs, args):
     utils.print_dict({'name': root[0], 'password': root[1]})
 
 
+@utils.arg('instance', metavar='<instance>',
+           help='ID or name of the instance.')
+@utils.service_type('database')
+def do_root_disable(cs, args):
+    """Disables root for an instance."""
+    instance = _find_instance(cs, args.instance)
+    cs.root.disable_instance_root(instance)
+
+
 @utils.arg('instance_or_cluster', metavar='<instance_or_cluster>',
            help='ID or name of the instance or cluster.')
 @utils.service_type('database')
@@ -1287,6 +1411,306 @@ def do_metadata_delete(cs, args):
     cs.metadata.delete(args.instance_id, args.key)
 
 
+@utils.arg('--datastore', metavar='<datastore>',
+           help='Name or ID of datastore to list modules for.')
+@utils.service_type('database')
+def do_module_list(cs, args):
+    """Lists the modules available."""
+    datastore = None
+    if args.datastore:
+        datastore = _find_datastore(cs, args.datastore)
+    module_list = cs.modules.list(datastore=datastore)
+    utils.print_list(
+        module_list,
+        ['id', 'tenant', 'name', 'type', 'datastore',
+         'datastore_version', 'auto_apply', 'visible'],
+        labels={'datastore_version': 'Version'})
+
+
+@utils.arg('module', metavar='<module>',
+           help='ID or name of the module.')
+@utils.service_type('database')
+def do_module_show(cs, args):
+    """Shows details of a module."""
+    module = _find_module(cs, args.module)
+    _print_object(module)
+
+
+@utils.arg('name', metavar='<name>', type=str, help='Name of the module.')
+@utils.arg('type', metavar='<type>', type=str,
+           help='Type of the module. The type must be supported by a '
+                'corresponding module plugin on the datastore it is '
+                'applied to.')
+@utils.arg('file', metavar='<filename>', type=argparse.FileType('rb', 0),
+           help='File containing data contents for the module.')
+@utils.arg('--description', metavar='<description>', type=str,
+           help='Description of the module.')
+@utils.arg('--datastore', metavar='<datastore>',
+           help='Name or ID of datastore this module can be applied to. '
+                'If not specified, module can be applied to all datastores.')
+@utils.arg('--datastore_version', metavar='<version>',
+           help='Name or ID of datastore version this module can be applied '
+                'to. If not specified, module can be applied to all versions.')
+@utils.arg('--auto_apply', action='store_true', default=False,
+           help='Automatically apply this module when creating an instance '
+                'or cluster.')
+@utils.arg('--all_tenants', action='store_true', default=False,
+           help='Module is valid for all tenants (Admin only).')
+# This option is to suppress the module from module-list for non-admin
+@utils.arg('--hidden', action='store_true', default=False,
+           help=argparse.SUPPRESS)
+@utils.arg('--live_update', action='store_true', default=False,
+           help='Allow module to be updated even if it is already applied '
+                'to a current instance or cluster. Automatically attempt to '
+                'reapply this module if the contents change.')
+@utils.service_type('database')
+def do_module_create(cs, args):
+    """Create a module."""
+
+    contents = args.file.read()
+    if not contents:
+        raise exceptions.ValidationError(
+            "The file '%s' must contain some data" % args.file)
+
+    module = cs.modules.create(
+        args.name, args.type, contents, description=args.description,
+        all_tenants=args.all_tenants, datastore=args.datastore,
+        datastore_version=args.datastore_version,
+        auto_apply=args.auto_apply, visible=not args.hidden,
+        live_update=args.live_update)
+    _print_object(module)
+
+
+@utils.arg('module', metavar='<module>', type=str,
+           help='Name or ID of the module.')
+@utils.arg('--name', metavar='<name>', type=str, default=None,
+           help='Name of the module.')
+@utils.arg('--type', metavar='<type>', type=str, default=None,
+           help='Type of the module. The type must be supported by a '
+                'corresponding module plugin on the datastore it is '
+                'applied to.')
+@utils.arg('--file', metavar='<filename>', type=argparse.FileType('rb', 0),
+           default=None,
+           help='File containing data contents for the module.')
+@utils.arg('--description', metavar='<description>', type=str, default=None,
+           help='Description of the module.')
+@utils.arg('--datastore', metavar='<datastore>',
+           help='Name or ID of datastore this module can be applied to. '
+                'If not specified, module can be applied to all datastores.')
+@utils.arg('--all_datastores', dest='datastore', action='store_const',
+           const=None,
+           help='Module is valid for all datastores.')
+@utils.arg('--datastore_version', metavar='<version>',
+           help='Name or ID of datastore version this module can be applied '
+                'to. If not specified, module can be applied to all versions.')
+@utils.arg('--all_datastore_versions', dest='datastore_version',
+           action='store_const', const=None,
+           help='Module is valid for all datastore version.')
+@utils.arg('--auto_apply', action='store_true', default=None,
+           help='Automatically apply this module when creating an instance '
+                'or cluster.')
+@utils.arg('--no_auto_apply', dest='auto_apply', action='store_false',
+           default=None,
+           help='Do not automatically apply this module when creating an '
+                'instance or cluster.')
+@utils.arg('--all_tenants', action='store_true', default=None,
+           help='Module is valid for all tenants (Admin only).')
+@utils.arg('--no_all_tenants', dest='all_tenants', action='store_false',
+           default=None,
+           help='Module is valid for current tenant only (Admin only).')
+# This option is to suppress the module from module-list for non-admin
+@utils.arg('--hidden', action='store_true', default=None,
+           help=argparse.SUPPRESS)
+# This option is to allow the module to be seen from module-list for non-admin
+@utils.arg('--no_hidden', dest='hidden', action='store_false', default=None,
+           help=argparse.SUPPRESS)
+@utils.arg('--live_update', action='store_true', default=None,
+           help='Allow module to be updated or deleted even if it is already '
+                'applied to a current instance or cluster. Automatically '
+                'attempt to reapply this module if the contents change.')
+@utils.arg('--no_live_update', dest='live_update', action='store_false',
+           default=None,
+           help='Restricts a module from being updated or deleted if it is '
+                'already applied to a current instance or cluster.')
+@utils.service_type('database')
+def do_module_update(cs, args):
+    """Create a module."""
+    module = _find_module(cs, args.module)
+    contents = args.file.read() if args.file else None
+    visible = not args.hidden if args.hidden is not None else None
+    datastore_args = {}
+    if args.datastore:
+        datastore_args['datastore'] = args.datastore
+    if args.datastore_version:
+        datastore_args['datastore_version'] = args.datastore_version
+    updated_module = cs.modules.update(
+        module, name=args.name, module_type=args.type, contents=contents,
+        description=args.description, all_tenants=args.all_tenants,
+        auto_apply=args.auto_apply, visible=visible,
+        live_update=args.live_update, **datastore_args)
+    _print_object(updated_module)
+
+
+@utils.arg('module', metavar='<module>',
+           help='ID or name of the module.')
+@utils.service_type('database')
+def do_module_delete(cs, args):
+    """Delete a module."""
+    module = _find_module(cs, args.module)
+    cs.modules.delete(module)
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.service_type('database')
+def do_log_list(cs, args):
+    """Lists the log files available for instance."""
+    instance = _find_instance(cs, args.instance)
+    log_list = cs.instances.log_list(instance)
+    utils.print_list(log_list,
+                     ['name', 'type', 'status', 'published', 'pending',
+                      'container', 'prefix'])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to show.')
+@utils.service_type('database')
+def do_log_show(cs, args):
+    """Instructs Trove guest to show details of log."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_info = cs.instances.log_show(instance, args.log_name)
+        _print_object(log_info)
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.service_type('database')
+def do_log_enable(cs, args):
+    """Instructs Trove guest to start collecting log details."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_info = cs.instances.log_enable(instance, args.log_name)
+        _print_object(log_info)
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.arg('--discard', action='store_true', default=False,
+           help='Discard published contents of specified log.')
+@utils.service_type('database')
+def do_log_disable(cs, args):
+    """Instructs Trove guest to stop collecting log details."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_info = cs.instances.log_disable(instance, args.log_name,
+                                            discard=args.discard)
+        _print_object(log_info)
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.arg('--disable', action='store_true', default=False,
+           help='Stop collection of specified log.')
+@utils.arg('--discard', action='store_true', default=False,
+           help='Discard published contents of specified log.')
+@utils.service_type('database')
+def do_log_publish(cs, args):
+    """Instructs Trove guest to publish latest log entries on instance."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_info = cs.instances.log_publish(
+            instance, args.log_name, disable=args.disable,
+            discard=args.discard)
+        _print_object(log_info)
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.service_type('database')
+def do_log_discard(cs, args):
+    """Instructs Trove guest to discard the container of the published log."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_info = cs.instances.log_discard(instance, args.log_name)
+        _print_object(log_info)
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.arg('--publish', action='store_true', default=False,
+           help='Publish latest entries from guest before display.')
+@utils.arg('--lines', metavar='<lines>', default=50, type=int,
+           help='Publish latest entries from guest before display.')
+@utils.service_type('database')
+def do_log_tail(cs, args):
+    """Display log entries for instance."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        log_gen = cs.instances.log_generator(instance, args.log_name,
+                                             args.publish, args.lines)
+        for log_part in log_gen():
+            print(log_part, end="")
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
+@utils.arg('instance', metavar='<instance>',
+           help='Id or Name of the instance.')
+@utils.arg('log_name', metavar='<log_name>', help='Name of log to publish.')
+@utils.arg('--publish', action='store_true', default=False,
+           help='Publish latest entries from guest before display.')
+@utils.arg('--file', metavar='<file>', default=None,
+           help='Path of file to save log to for instance.')
+@utils.service_type('database')
+def do_log_save(cs, args):
+    """Save log file for instance."""
+    try:
+        instance = _find_instance(cs, args.instance)
+        filename = cs.instances.log_save(instance, args.log_name,
+                                         args.publish, args.file)
+        print('Log "%s" written to %s' % (args.log_name, filename))
+    except exceptions.GuestLogNotFoundError:
+        print(NO_LOG_FOUND_ERROR % (args.log_name, instance))
+    except Exception as ex:
+        error_msg = ex.message.split('\n')
+        print(error_msg[0])
+
+
 # @utils.arg('datastore_version',
 #            metavar='<datastore_version>',
 #            help='Datastore version name or UUID assigned to the '
